#include "SPI_Definitions.h"
#include "ccRF_3_registers.h"

char ccRF_SPICmdStrobe(char cmd){
    char rc;
    RC2_bit = 0;           // TRXEM_SPI_BEGIN();
    asm NOP;
    rc = SPI3_Read(cmd);   //  TRXEM_SPI_TX(cmd);
                           //  TRXEM_SPI_WAIT_DONE();
                           //  rc = TRXEM_SPI_RX();
    RC2_bit = 1;           // TRXEM_SPI_END();
    asm NOP;
    return(rc);
}
// ---------------------------------------

void ccRF_ReadWriteBurstSingle(char addr,char *pData,unsigned len)
{
  unsigned i;
  /* Communicate len number of bytes: if RX - the procedure sends 0x00 to push bytes from slave*/
  if(addr & RADIO_READ_ACCESS)
  {
    if(addr & RADIO_BURST_ACCESS)
    {
      for (i = 0; i < len; i++)
      {
          *pData++ = SPI3_Read(0);            /* Possible to combining read and write as one access type */
      }
    }
    else
    {
      *pData = SPI3_Read(0);
    }
  }
  else
  {
    if(addr&RADIO_BURST_ACCESS)
    {
      /* Communicate len number of bytes: if TX - the procedure doesn't overwrite pData */
      for (i = 0; i < len; i++)
      {
        SPI3_Write(*pData++);
      }
    }
    else
    {
      SPI3_Write(*pData);
    }
  }
  return;
}

char ccRF_8BitRegAccess(char accessType, char addrByte, char *pData, unsigned len)
{
  char readValue;
  char txtv[5];

  /* Pull CS_N low and wait for SO to go low before communication starts */
  RC2_bit = 0;
  asm NOP;
  /* send register address byte */
  readValue = SPI3_Read(accessType|addrByte);
  ccRF_ReadWriteBurstSingle(accessType|addrByte, pData, len);
  asm NOP;
  RC2_bit = 1;
  /* return the status byte value */
  return(readValue);
}

char ccRF_16BitRegAccess(char accessType, char extAddr, char regAddr, char *pData, char len)
{
  char readValue;
  RC2_bit = 0;
  asm NOP;

  /* send extended address byte with access type bits set */
  readValue = SPI3_Read(accessType|extAddr);
  SPI3_Write(regAddr);
  /* Communicate len number of bytes */
  ccRF_ReadWriteBurstSingle(accessType|extAddr, pData, len);
  asm NOP;
  RC2_bit = 1;
  /* return the status byte value */
  return(readValue);
}

char ccRF_SPIWriteReg(unsigned addr, char *pData, char len){
  char tempExt  = (char)(addr>>8);
  char tempAddr = (char)(addr & 0x00FF);
  char rc;

  /* Checking if this is a FIFO access - returns chip not ready */
  if((CCRF_3_SINGLE_TXFIFO <= tempAddr) && (tempExt == 0)) return STATUS_CHIP_RDYn_BM;

  /* Decide what register space is accessed */
  if(!tempExt){
    rc = ccRF_8BitRegAccess((RADIO_BURST_ACCESS|RADIO_WRITE_ACCESS), tempAddr, pData, len);
  }
  else if (tempExt == 0x2F){
    rc = ccRF_16BitRegAccess((RADIO_BURST_ACCESS|RADIO_WRITE_ACCESS), tempExt, tempAddr, pData, len);
  }
  return (rc);
}

char ccRF_SPIReadReg(unsigned addr, char *pData, char len)
{
  char tempExt  = (char)(addr>>8);
  char tempAddr = (char)(addr & 0x00FF);
  char rc;

  /* Checking if this is a FIFO access -> returns chip not ready  */
  if((CCRF_3_SINGLE_TXFIFO<=tempAddr)&&(tempExt==0)) return STATUS_CHIP_RDYn_BM;

  /* Decide what register space is accessed */
  if(!tempExt)
  {
    rc = ccRF_8BitRegAccess((RADIO_BURST_ACCESS|RADIO_READ_ACCESS),tempAddr,pData,len);
  }
  else if (tempExt == 0x2F)
  {
    rc = ccRF_16BitRegAccess((RADIO_BURST_ACCESS|RADIO_READ_ACCESS),tempExt,tempAddr,pData,len);
  }
  return (rc);
}

char ccRF_SPIWriteTxFifo(char *pData, char len)
{
  char rc;
  rc = ccRF_8BitRegAccess(0x00,CCRF_3_BURST_TXFIFO, pData, len);
  return (rc);
}

char ccRF_SPIReadRxFifo(char * pData, char len)
{
  char rc;
  rc = ccRF_8BitRegAccess(0x00,CCRF_3_BURST_RXFIFO, pData, len);
  return (rc);
}

char ccRF_ReadGPIO2(){
  return RD0_bit;
}