#include "easy_link_reg_config.h"
#include "SPI_Functions.h"

#define PKTLEN                  5  // 1 < PKTLEN < 126

extern sfr sbit CCRF_3_GP2;
extern sfr sbit CCRF_3_GP3;
volatile unsigned packetCounter   = 0;
volatile unsigned receivedCounter = 0;
char buffer[PKTLEN+1] = {0};

void ccRF_CreatePacket(char txBuffer[])
{
  int i;
  char tmp = 0xAA;
  buffer[0] = PKTLEN;                          // Length byte
  buffer[1] = (char) (packetCounter >> 8);     // MSB of packetCounter
  buffer[2] = (char)  packetCounter;           // LSB of packetCounter

  // Fill rest of buffer with random bytes
  for(i = 3; i < PKTLEN; i++) 
  {
    if ( i % 2 )
    {
     buffer[i] = tmp;
    }
    else
    {
     buffer[i] = ~tmp;
    }
  }
}

void ccRF_RunTX()
{
char txt_cnt[5];
  // Update packet counter
  packetCounter++;
  WordToStr(packetCounter, txt_cnt);
  TFT_Rectangle(180,120,280,150);
  TFT_Write_Text(txt_cnt, 200, 130);

  // Create a random packet with PKTLEN + 2 byte packet
  // counter + n x random bytes
  ccRF_CreatePacket(buffer);
  // Write packet to TX FIFO
  ccRF_SPIWriteTxFifo(buffer, sizeof(buffer));
  // Strobe TX to send packet
  ccRF_SPICmdStrobe(CCRF_3_STX);
  Delay_100ms();
}

#define RX_FIFO_ERROR       0x11
void ccRF_SetRXmode()
{
  ccRF_SpiCmdStrobe(CCRF_3_SRX);
}

void ccRF_RunRX(){
char txt[25];
unsigned char rxBytes;
char marcState;
unsigned receivedCnt = 0;

   if(CCRF_3_GP3){
    while(CCRF_3_GP3);
    
    // Read number of bytes in RX FIFO
    ccRF_SPIReadReg(CCRF_3_NUM_RXBYTES, &rxBytes, 1);
    // Check that we have bytes in FIFO
    if(rxBytes != 0) {
        // Read MARCSTATE to check for RX FIFO error
        ccRF_SPIReadReg(CCRF_3_MARCSTATE, &marcState, 1);
        // Mask out MARCSTATE bits and check if we have a RX FIFO error
        if((marcState & 0x1F) == RX_FIFO_ERROR) {
            // Flush RX FIFO
            CCRF_SpiCmdStrobe(CCRF_3_SFRX);
            TFT_Write_Text("ERROR", 200, 180);
        } else {
            // Read n bytes from RX FIFO
            ccRF_SPIReadRxFifo(buffer, rxBytes);
            // Check CRC ok (CRC_OK: bit7 in second status byte)
            // This assumes status bytes are appended in RX_FIFO
            // (PKT_CFG1.APPEND_STATUS = 1)
            // If CRC is disabled the CRC_OK field will read 1
            if(buffer[rxBytes - 1] & 0x80) {
                ccRF_SetRXmode();
                // Update packet counter
                receivedCounter++;
                receivedCnt  = buffer[1]<<8;
                receivedCnt += buffer[2];
                WordToStr(receivedCnt, txt);
                TFT_Set_Pen( CL_WHITE, 1 );
                TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
                TFT_Rectangle(190, 180, 300, 200);
                TFT_Write_Text(txt, 200, 180);

            }
        }
    }
  }
}

void ccRF_RegisterConfig(void)
{
  unsigned char writeByte;
  unsigned i;
  // Reset radio
  CCRF_SpiCmdStrobe(CCRF_3_SRES);

  // Write registers to radio
  for(i = 0; i < (sizeof(preferredSettings_433)/sizeof(registerSetting_t)); i++) 
  {
    writeByte = preferredSettings_433[i].dat;
    CCRF_SpiWriteReg(preferredSettings_433[i].addr, &writeByte, 1);
  }
}

#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2

void ccRF_ManualCalibration(void) {

    char original_fs_cal2;
    char calResults_for_vcdac_start_high[3];
    char calResults_for_vcdac_start_mid[3];
    char marcstate;
    char writeByte;

    // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
    writeByte = 0x00;
    CCRF_SpiWriteReg(CCRF_3_FS_VCO2, &writeByte, 1);

    // 2) Start with high VCDAC (original VCDAC_START + 2):
    CCRF_SpiReadReg(CCRF_3_FS_CAL2, &original_fs_cal2, 1);
    writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
    CCRF_SpiWriteReg(CCRF_3_FS_CAL2, &writeByte, 1);

    // 3) Calibrate and wait for calibration to be done
    //   (radio back in IDLE state)
    CCRF_SpiCmdStrobe(CCRF_3_SCAL);

    do {
        CCRF_SpiReadReg(CCRF_3_MARCSTATE, &marcstate, 1);
    } while (marcstate != 0x41);

    // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with
    //    high VCDAC_START value
    CCRF_SpiReadReg(CCRF_3_FS_VCO2,
                     &calResults_for_vcdac_start_high[FS_VCO2_INDEX], 1);
    CCRF_SpiReadReg(CCRF_3_FS_VCO4,
                     &calResults_for_vcdac_start_high[FS_VCO4_INDEX], 1);
    CCRF_SpiReadReg(CCRF_3_FS_CHP,
                     &calResults_for_vcdac_start_high[FS_CHP_INDEX], 1);

    // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
    writeByte = 0x00;
    CCRF_SpiWriteReg(CCRF_3_FS_VCO2, &writeByte, 1);

    // 6) Continue with mid VCDAC (original VCDAC_START):
    writeByte = original_fs_cal2;
    CCRF_SpiWriteReg(CCRF_3_FS_CAL2, &writeByte, 1);

    // 7) Calibrate and wait for calibration to be done
    //   (radio back in IDLE state)
    CCRF_SpiCmdStrobe(CCRF_3_SCAL);

    do {
        CCRF_SpiReadReg(CCRF_3_MARCSTATE, &marcstate, 1);
    } while (marcstate != 0x41);

    // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained
    //    with mid VCDAC_START value
    CCRF_SpiReadReg(CCRF_3_FS_VCO2,
                     &calResults_for_vcdac_start_mid[FS_VCO2_INDEX], 1);
    CCRF_SpiReadReg(CCRF_3_FS_VCO4,
                     &calResults_for_vcdac_start_mid[FS_VCO4_INDEX], 1);
    CCRF_SpiReadReg(CCRF_3_FS_CHP,
                     &calResults_for_vcdac_start_mid[FS_CHP_INDEX], 1);

    // 9) Write back highest FS_VCO2 and corresponding FS_VCO
    //    and FS_CHP result
    if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] >
        calResults_for_vcdac_start_mid[FS_VCO2_INDEX]) {
        writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
        CCRF_SpiWriteReg(CCRF_3_FS_VCO2, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
        CCRF_SpiWriteReg(CCRF_3_FS_VCO4, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
        CCRF_SpiWriteReg(CCRF_3_FS_CHP, &writeByte, 1);
    } else {
        writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
        CCRF_SpiWriteReg(CCRF_3_FS_VCO2, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
        CCRF_SpiWriteReg(CCRF_3_FS_VCO4, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
        CCRF_SpiWriteReg(CCRF_3_FS_CHP, &writeByte, 1);
    }
}

void ccRF_Configure(){
  ccRF_RegisterConfig();
  ccRF_ManualCalibration();
}